    /** This section is for TinyFugue emulated &quot;/" commands
     * like TinyFugue uses
     * $Id: JMTFCommands.java,v 1.7 2004/02/01 03:28:01 jeffnik Exp $
     */

package anecho.JamochaMUD;

import java.util.StringTokenizer;
import java.util.Vector;

public class JMTFCommands {

    private JMConfig settings;
    private String ESC = "\u001b";

    public JMTFCommands(JMConfig settings) {
        this.settings = settings;
    }

    public boolean Command(String command) {
        boolean response = false;       // return-code becomes true if we do process a command
        String workString, cmd, tempFlag;
        StringTokenizer tokens;
        Vector options = new Vector(0, 1);        // contain any flags that may be present on the command
        Vector args = new Vector(0, 1);

        workString = command.toLowerCase();

        int space = workString.indexOf(' ');

        if (space < 0) {
            // Single word command
            cmd = workString;
        } else {
            cmd = workString.substring(0, space);
        }

        tokens = new StringTokenizer(command);

        int total = tokens.countTokens();

        // Parse apart our arguments and options
        for (int i = 0; i < total; i++) {
            tempFlag = tokens.nextToken().trim();
            if (tempFlag.startsWith("-")) {
                options.addElement(tempFlag);
            } else {
                args.addElement(tempFlag);
            }
        }

        if (cmd.equals("/connect")) {
            connectToWorld(args, options);
            response = true;
        }

        if (cmd.equals("/localecho")) {
            localEcho(args);
            response = true;
        }

        if (cmd.equals("/set")) {
            setVariable(command, args, options);
            response = true;
        }

        if (cmd.equals("/unset")) {
            unsetVariable(command);
            response = true;
        }

        if (cmd.equals("/def")) {
            makeDefinition(command, args, options);
            response = true;
        }

        if (cmd.equals("/undef")) {
            unsetDefinition(command);
            response = true;
        }

        if (!response) {
            response = userDefinedCommand(command,
                                          options,
                                          args);
        }

        return response;
    }

    private void connectToWorld(Vector userArgs, Vector options) {
        System.out.println("Connect to " + userArgs.elementAt(1) + ":" + userArgs.elementAt(2));
    }

    private void localEcho(Vector userArgs) {

    }

    private void setVariable(String command, Vector args, Vector options) {
        String name, value;
        int eqSign = command.indexOf("=");

        if (command.length() <= eqSign) {
            return;
        }

        // if (command.toLowerCase().trim().equals("/set")) {
        if (args.size() == 1) {
            // Display all of our set items
            if (options.size() > 0) {
                displaySettingsWindow(settings.getAllVariables());
            } else {
                displaySettings(settings.getAllVariables());
            }
            return;
        }

        if (eqSign < 0 && command.indexOf(" ") > 0) {
            // Show the user the value of the given variable
            name = command.substring(4).trim();
            value = settings.getVariable(name);

            if (value != null) {
                // text.append("Variable " + ESC + "[1m" + name + "" + ESC + "[0m is set to " + ESC + "[1m" + value + "" + ESC + "[0m\n");
                sendUserMessage("Variable " + ESC + "[1m" + name + ESC + "[0m is set to " + ESC + "[1m" + value + ESC + "[0m\n");
            } else {
                // text.append("Variable " + ESC + "[1m" + name + "" + ESC + "[0m is not set\n");
                sendUserMessage("Variable " + ESC + "[1m" + name + "" + ESC + "[0m is not set\n");
            }

            return;
        }

        name = command.substring(4, eqSign).trim();
        value = command.substring(eqSign + 1).trim();

        settings.addVariable(name, value);
        sendUserMessage("Variable " + ESC + "[1m" + name + "" + ESC + "[0m is now set to " + ESC + "[1m" + value + "" + ESC + "[0m\n");

    }

    private void unsetVariable(String command) {
        if (command.length() < 6) {
            return;
        }

        String name = command.substring(6).trim();

        settings.removeVariable(name);
        sendUserMessage("Variable " + ESC + "[1m" + name + "" + ESC + "[0m has been unset\n");

    }

    private void makeDefinition(String command, Vector args, Vector options) {
        String name, value;
        int eqSign;

        // Initially we are going to ignore the args
        eqSign = command.indexOf("=");

        if (command.length() <= eqSign) {
            return;
        }

        if (args.size() == 1) {
            // Display all the definitions
            if (options.size() > 0) {
                displaySettingsWindow(settings.getAllDefinitions());
            } else {
                displaySettings(settings.getAllDefinitions());
            }
            return;
        }

        // if (eqSign < 0 || eqSign >= command.length()) {
        if (eqSign < 0 && command.indexOf(" ") > 0) {
            // return;
            // }

            // if (args.size() > 0) {
            // name = command.substring(5, eqSign).trim();
            name = command.substring(4).trim();
            // } else {
            value = settings.getDefinition(name);

            if (value != null) {
                // text.append("Variable " + ESC + "[1m" + name + "" + ESC + "[0m is set to " + ESC + "[1m" + value + "" + ESC + "[0m\n");
                sendUserMessage("Definition " + ESC + "[1m" + name + "" + ESC + "[0m is set to " + ESC + "[1m" + value + "" + ESC + "[0m\n");
            } else {
                // text.append("Variable " + ESC + "[1m" + name + "" + ESC + "[0m is not set\n");
                sendUserMessage("Definition " + ESC + "[1m" + name + "" + ESC + "[0m is not set\n");
            }

            return;
        }

        name = command.substring(5, eqSign).trim();
        value = command.substring(eqSign + 1);

        settings.addDefinition(name, value);

        // CHandler connHandler = settings.getConnectionHandler();
        // anecho.gui.JMText text;
        // text = connHandler.getActiveMUDText();
        // text.append("Definition " + ESC + "[1m" + name + "" + ESC + "[0m set to " + ESC + "[1m" + value + "" + ESC + "[0m\n");
        sendUserMessage("Definition " + ESC + "[1m" + name + "" + ESC + "[0m set to " + ESC + "[1m" + value + "" + ESC + "[0m\n");

    }

    private boolean userDefinedCommand(String command, Vector options, Vector args) {
        String name, value, temp;
        boolean expand = true;
        int start, end;

        temp = args.elementAt(0).toString();
        name = temp.substring(1);

        value = settings.getDefinition(name);

        if (value == null) {
            // System.out.println("No value, sending initial command");
            DataIn sender = settings.getDataInVariable();
            // sender.JMSendText(value);
            // sender.JMSendText();

            return true;
        }

        // Now parse our new value for any arguments that need to be translated

        // We're going after anything that looks like %1 or %{1-something}
        while (expand) {
            start = value.indexOf("%");

            if (start + 1 >= value.length()) {
                start = -1;
                expand = false;
            }

            if (start > -1 && expand) {
                String fillToken = "";
                int replace;

                if (value.charAt(start + 1) == '{') {
                    // Complex replacement
                    end = start + 2;
                    while (end < value.length() && Character.isDigit(value.charAt(end))) {
                        end++;
                    }

                    replace = Integer.parseInt(value.substring(start + 2, end));

                    // Now figure out what our default replacement token will be
                    if (value.charAt(end) == '-') {
                        int startDash = end + 1;
                        int endBracket = value.indexOf("}", end);

                        if (endBracket > -1 && endBracket <= value.length()) {

                            fillToken = value.substring(startDash, endBracket);
                            end = endBracket + 1;
                        } else {
                            end++;
                        }
                    } else {
                        replace = args.size() + 1;
                        end = start + 1;
                    }



                } else {
                    end = start + 1;
                    // while (end <= value.length() && ((Character)(value.charAt(end))).isDigit()) {
                    while (end < value.length() && Character.isDigit(value.charAt(end))) {
                        end++;
                    }


                    // Figure out replacement
                    replace = Integer.parseInt(value.substring(start + 1, end));

                }

                // if (replace < args.size()) {
                String startS, endS, repValue;
                if (start > 0) {
                    startS = value.substring(0, start);
                } else {
                    startS = "";
                }
                if (end < value.length()) {
                    endS = value.substring(end);
                } else {
                    endS = "";
                }

                if (replace < args.size()) {
                    repValue = (String)args.elementAt(replace);
                } else {
                    // repValue = "*";
                    repValue = fillToken;
                }

                value = startS + repValue + endS;

            } else {
                expand = false;
            }

        }

        // Send the new value via DataIn
        DataIn sender = settings.getDataInVariable();
        sender.JMSendText(value);

        return true;

    }

    private void unsetDefinition(String command) {
        if (command.length() < 6) {
            return;
        }

        String name = command.substring(6).trim();

        settings.removeDefinition(name);
        sendUserMessage("Definition " + ESC + "[1m" + name + "" + ESC + "[0m has been undefined\n");
    }

    /** Writes a message back to the user (usually a confirmation
     * on the active MU* that is showing
     */
    private void sendUserMessage(String message) {
        CHandler connHandler = settings.getConnectionHandler();

        if (settings.getJMboolean(JMConfig.USESWING)) {
            anecho.gui.JMSwingText text;
            text = connHandler.getActiveMUDSwingText();
            text.append(message);
        } else {
            anecho.gui.JMText text;
            text = connHandler.getActiveMUDText();
            text.append(message);
        }

    }

    /** Display a new dialogue that lists all of our definitions in the hashtable
     */
    private void displaySettingsWindow(java.util.Hashtable userDefs) {
        String name, value, tempLine;
        int longLine = 0;

        anecho.gui.SyncFrame defFrame = new anecho.gui.SyncFrame("JamochaMUD");
        java.awt.TextArea defs = new java.awt.TextArea("", 20, 20, java.awt.TextArea.SCROLLBARS_VERTICAL_ONLY);
        defs.setEditable(false);
        defFrame.add(defs);

        // Iternate through our hashtable and show all our settings:
        java.util.Enumeration keys = userDefs.keys();

        while (keys.hasMoreElements()) {
            name = (String)keys.nextElement();
            value = userDefs.get(name).toString();
            tempLine = name + " = " + value + '\n';
            // defs.append(name + " = " + value + '\n');
            if (tempLine.length() > longLine) {
                longLine = tempLine.length();
            }
            defs.append(tempLine);
        }

        defs.setColumns(longLine);

        defFrame.setCloseState(true);
        defFrame.pack();
        defFrame.setVisible(true);
    }

    /** Write the hashtable out to the current MU*
     */
    private void displaySettings(java.util.Hashtable userDefs) {
        String name, value;
        // CHandler connHandler = settings.getConnectionHandler();
        // anecho.gui.JMText text;
        // text = connHandler.getActiveMUDText();

        java.util.Enumeration keys = userDefs.keys();

        while (keys.hasMoreElements()) {
            name = (String)keys.nextElement();
            value = userDefs.get(name).toString();
            // text.append("" + ESC + "[1m" + name + "" + ESC + "[0m = " + ESC + "[1m" + value + "" + ESC + "[0m\n");
            sendUserMessage("" + ESC + "[1m" + name + "" + ESC + "[0m = " + ESC + "[1m" + value + "" + ESC + "[0m\n");
        }
    }
}



