/** $Id: MainMenu.java,v 1.2 2004/07/28 02:47:48 jeffnik Exp $
 */
package anecho.JamochaMUD.legacy;

import java.awt.CheckboxMenuItem;
import java.awt.Frame;
import java.awt.MenuBar;
import java.awt.Menu;
import java.awt.MenuItem;
import java.awt.MenuShortcut;

import java.awt.event.KeyEvent;

import anecho.gui.ResReader;
import anecho.JamochaMUD.*;

/** Create a menu for the Main Window */
public class MainMenu {

    private static MainMenu _instance;  // We'll try playing with a Singleton!

    private MenuBar tWMenuBar;
    private CheckboxMenuItem tWMacro, tWSyncWindowsItem, tWUseUnicodeItem, tWTFKeysItem, tWLocalEchoItem, tWDoubleBufferItem, splitFramesItem;
    private CheckboxMenuItem tWAutoFocus, tWTimers, tWReleasePauseItem, tWAltFocus;
    private Menu tWMUListMenu;
    private MenuItem cTM, dFM, rTM, closeMU;
    private MenuItem dumpOutputItem, quitItem;
    private MenuItem pingMUItem;
    private MenuItem copyItem, pasteItem, findItem;
    private MenuItem coloursItem, externalProgramsItem, managePlugIn, serverOptionsItem;
    private MenuItem installPlugIn, removePlugIn;
    private MenuItem contentsItem, troubleshootingItem, aboutJamochaMUDItem;
    private JMConfig settings;
    protected static Menu tWPlugInMenu;

    private static final int COMBINED = 0;     // Combined frame style
    private static final int SPLIT = 1;        // Split frame style

    private int viewStyle;

    private ResReader reader;

    private static final boolean DEBUG = false;

    private MainMenu() {
        reader = new ResReader();
    }

    // For lazy initialization
    public static synchronized MainMenu getInstance() {
        if (_instance==null) {
            _instance = new MainMenu();
        }
        return _instance;
    }

    public void buildMenu(Frame parent, MuckMain mainProg, JMConfig settings) {
        this.settings = settings;
        tWMenuBar = new MenuBar();

        parent.setMenuBar(tWMenuBar);

        Menu tWFileMenu = new Menu(RB("file"));
        tWMenuBar.add(tWFileMenu);
        tWFileMenu.add(dumpOutputItem = new MenuItem(RB("dumpOutput"), new MenuShortcut(KeyEvent.VK_D, false)));
        dumpOutputItem.setActionCommand(dumpOutputItem.getLabel());

        tWFileMenu.addSeparator();
        tWFileMenu.add(quitItem = new MenuItem(RB("quit"), new MenuShortcut(KeyEvent.VK_Q, true)));
        quitItem.setActionCommand(quitItem.getLabel());
        tWFileMenu.addActionListener(mainProg);

        // Add the Edit menu items
        Menu tWEditMenu = new Menu(RB("edit"));
        tWMenuBar.add(tWEditMenu);
        // I'm making it the right key for OS/2, damn't!
        // if (settings.getOSName().equals("OS/2")) {
        if ((settings.getJMString(JMConfig.OSNAME)).equals("OS/2")) {
        // if (settings.getJMString(settings.OSNAME).equals("OS/2")) {
            copyItem = new MenuItem(RB("copyFromMainWindow"), new MenuShortcut(KeyEvent.VK_INSERT, false));
        } else {
            copyItem = new MenuItem(RB("copyFromMainWindow"), new MenuShortcut(KeyEvent.VK_C, false));
        }
        copyItem.setActionCommand(copyItem.getLabel());
        tWEditMenu.add(copyItem);

        pasteItem = new MenuItem(RB("paste"), new MenuShortcut(KeyEvent.VK_V, false));
        pasteItem.setActionCommand(pasteItem.getLabel());
        tWEditMenu.add(pasteItem);

        tWEditMenu.addSeparator();

        findItem = new MenuItem(RB("find"), new MenuShortcut(KeyEvent.VK_F, false));
        findItem.setActionCommand(findItem.getLabel());
        findItem.setEnabled(false);
        tWEditMenu.add(findItem);
        tWEditMenu.addActionListener(mainProg);


        // Add CONNECTION menu items
        Menu tWConnectMenu = new Menu(RB("connection"));
        tWMenuBar.add(tWConnectMenu);
        cTM = new MenuItem(RB("connectToMU"), new MenuShortcut(KeyEvent.VK_C, true)); // enable/disable functionality
        cTM.setActionCommand(cTM.getLabel());
        tWConnectMenu.add(cTM);

        rTM = new MenuItem(RB("reconnectToMU"), new MenuShortcut(KeyEvent.VK_R, true));
        rTM.setActionCommand(rTM.getLabel());
        tWConnectMenu.add(rTM);

        dFM = new MenuItem(RB("disconnectFromMU"), new MenuShortcut(KeyEvent.VK_D, true));  // enable/disable functionality
        dFM.setActionCommand(dFM.getLabel());
        dFM.setEnabled(false);
        tWConnectMenu.add(dFM);

        closeMU = new MenuItem(RB("closeThisView"));
        closeMU.setEnabled(false);
        tWConnectMenu.add(closeMU);

        tWConnectMenu.add(pingMUItem = new MenuItem(RB("pingMU")));
        pingMUItem.setEnabled(false);
        tWConnectMenu.addActionListener(mainProg);

        Menu tWOptionMenu = new Menu(RB("options"));
        tWMenuBar.add(tWOptionMenu);

        // Add ****Edit Options**** Submenu
        Menu tWEditOptionsMenu = new Menu(RB("editOptions"));
        tWOptionMenu.add(tWEditOptionsMenu);
        tWEditOptionsMenu.add(externalProgramsItem = new MenuItem(RB("externalPrograms"), new MenuShortcut(KeyEvent.VK_E, false)));
        externalProgramsItem.setActionCommand(externalProgramsItem.getLabel());
        // externalProgramsItem.setEnabled(false);

        tWEditOptionsMenu.add(managePlugIn = new MenuItem(RB("ManagePlugins.title"), new MenuShortcut(KeyEvent.VK_M, false)));
        managePlugIn.setActionCommand(managePlugIn.getLabel());
        tWEditOptionsMenu.add(serverOptionsItem = new MenuItem(RB("serverOptions")));
        tWEditOptionsMenu.addActionListener(mainProg);

        installPlugIn = new MenuItem(RB("installPlugin"));
        installPlugIn.setActionCommand(installPlugIn.getLabel());

        removePlugIn = new MenuItem(RB("removePlugin"));
        removePlugIn.setEnabled(false);
        removePlugIn.setActionCommand(removePlugIn.getLabel());

        tWOptionMenu.add(coloursItem = new MenuItem(RB("fontsAndColours"), new MenuShortcut(KeyEvent.VK_F, true)));
        coloursItem.setActionCommand(coloursItem.getLabel());

        // Continue with normal option menu
        tWOptionMenu.addSeparator();

        tWMacro = new CheckboxMenuItem(RB("showMacroBar"), false);
        tWOptionMenu.add(tWMacro);
        tWMacro.addItemListener(mainProg);
        tWMacro.setEnabled(false);

        tWTimers = new CheckboxMenuItem(RB("showTimers"), false);
        tWTimers.setShortcut(new MenuShortcut(KeyEvent.VK_T, false));
        tWTimers.setActionCommand(tWTimers.getLabel());
        tWOptionMenu.add(tWTimers);
        tWTimers.addItemListener(mainProg);
        tWTimers.addActionListener(mainProg);
        // Fix mainProg XXX!!  Right now we have no option for timers at all
        tWTimers.setEnabled(false);

        splitFramesItem = new CheckboxMenuItem(RB("splitFrames"), false);
        tWOptionMenu.add(splitFramesItem);
        // if (settings.getSplitView()) {
        if (settings.getJMboolean(JMConfig.SPLITVIEW)) {
            viewStyle = SPLIT;
            splitFramesItem.setState(true);
        } else {
            viewStyle = COMBINED;
        }
        splitFramesItem.addItemListener(mainProg);

        tWSyncWindowsItem = new CheckboxMenuItem(RB("syncWindows"), true);
        tWOptionMenu.add(tWSyncWindowsItem);
        tWSyncWindowsItem.addItemListener(mainProg);
        // tWSyncWindowsItem.setState(settings.getSyncWindows());
        tWSyncWindowsItem.setState(settings.getJMboolean(JMConfig.SYNCWINDOWS));

        tWOptionMenu.addSeparator();

        tWAutoFocus = new CheckboxMenuItem(RB("autoFocusInput"), true);
        tWOptionMenu.add(tWAutoFocus);
        tWAutoFocus.addItemListener(mainProg);
        // tWAutoFocus.setState(settings.getAutoFocusInput());
        tWAutoFocus.setState(settings.getJMboolean(settings.AUTOFOCUSINPUT));

        tWAltFocus = new CheckboxMenuItem(RB("altFocus"), true);
        tWOptionMenu.add(tWAltFocus);
        tWAltFocus.addItemListener(mainProg);
        // tWAltFocus.setState(settings.getAltFocus());
        tWAltFocus.setState(settings.getJMboolean(JMConfig.ALTFOCUS));
        if (!splitFramesItem.getState()) {
            tWAltFocus.setEnabled(false);
        }

        tWLocalEchoItem = new CheckboxMenuItem(RB("localEcho"), true);
        tWOptionMenu.add(tWLocalEchoItem);
        tWLocalEchoItem.addItemListener(mainProg);
        // tWLocalEchoItem.setState(settings.isLocalEchoEnabled());
        tWLocalEchoItem.setState(settings.getJMboolean(JMConfig.LOCALECHO));

        tWDoubleBufferItem = new CheckboxMenuItem(RB("doubleBuffer"), true);
        tWOptionMenu.add(tWDoubleBufferItem);
        tWDoubleBufferItem.addItemListener(mainProg);
        // tWDoubleBufferItem.setState(settings.getDoubleBuffer());
        tWDoubleBufferItem.setState(settings.getJMboolean(JMConfig.DOUBLEBUFFER));

        tWReleasePauseItem = new CheckboxMenuItem(RB("releasePause"), true);
        tWOptionMenu.add(tWReleasePauseItem);
        tWReleasePauseItem.addItemListener(mainProg);
        // tWReleasePauseItem.setState(settings.getReleasePause());
        tWReleasePauseItem.setState(settings.getJMboolean(JMConfig.RELEASEPAUSE));

        tWTFKeysItem = new CheckboxMenuItem(RB("tinyFugueKeys"), true);
        tWOptionMenu.add(tWTFKeysItem);
        tWTFKeysItem.addItemListener(mainProg);
        // tWTFKeysItem.setState(settings.getTFKeyEmu());
        tWTFKeysItem.setState(settings.getJMboolean(settings.TFKEYEMU));

        tWUseUnicodeItem = new CheckboxMenuItem(RB("useUnicode"), false);
        tWOptionMenu.add(tWUseUnicodeItem);
        tWUseUnicodeItem.addItemListener(mainProg);
        // tWUseUnicodeItem.setState(settings.getUseUnicode());
        tWUseUnicodeItem.setState(settings.getJMboolean(settings.USEUNICODE));

        // Add the ActionListener to mainProg menu
        tWOptionMenu.addActionListener(mainProg);


        // Add the Plug-in menu items
        tWPlugInMenu = new Menu(RB("plugIn"));
        tWMenuBar.add(tWPlugInMenu);
        tWPlugInMenu.addActionListener(mainProg);

        // List for our active MU*s
        tWMUListMenu = new Menu(RB("MUList"));
        tWMenuBar.add(tWMUListMenu);
        tWMUListMenu.addActionListener(mainProg);

        //Add HELP menu items
        Menu tWHelpMenu = new Menu(RB("help"));
        tWMenuBar.setHelpMenu(tWHelpMenu);
        tWHelpMenu.add(contentsItem = new MenuItem(RB("contents"), new MenuShortcut(KeyEvent.VK_H)));
        contentsItem.setActionCommand(contentsItem.getLabel());
        tWHelpMenu.add(troubleshootingItem = new MenuItem(RB("reportABug")));
        tWHelpMenu.add(new MenuItem(RB("tinyFugueKeyCodes")));
        // tWHelpMenu.add(troubleshootingItem = new MenuItem(RB("troubleshooting")));
        tWHelpMenu.addSeparator();
        tWHelpMenu.add(aboutJamochaMUDItem = new MenuItem(RB("aboutJamochaMUD")));
        tWHelpMenu.addActionListener(mainProg);

    }

    public synchronized void setCloseMUEnabled(boolean state) {
        closeMU.setEnabled(state);
    }

    public void removeAllPlugins() {
        tWPlugInMenu.removeAll();
        // tWPlugInMenu.add(managePlugIn);
        tWPlugInMenu.add(installPlugIn);
        tWPlugInMenu.add(removePlugIn);
        tWPlugInMenu.addSeparator();
    }

    // public void addPlugin(String name, boolean state) {
    public void addPlugin(anecho.JamochaMUD.plugins.PlugInterface plug) {

        if (DEBUG) {
            System.err.println("MainMenu adding plug-in: " + plug.PlugInName());
        }

        String name = plug.PlugInName();
        boolean state = plug.isActive();
        boolean properties = plug.HasProperties();

        Menu tempName = new Menu(name);
        /*
        if (state) {
            tempName.setIcon(new ImageIcon(JMainMenu.class.getResource("icons/22/connect_established.png")));
        } else {
            tempName.setIcon(new ImageIcon(JMainMenu.class.getResource("icons/22/connect_no.png")));
        }
        */

        MenuItem prop = new MenuItem(RB("properties"));
        // prop.setIcon(new ImageIcon(JMainMenu.class.getResource("icons/22/configure.png")));

        if (!properties) {
            // No settings are available for this item
            prop.setEnabled(false);
        } else {
            prop.setActionCommand("plugin:" + name);
            prop.addActionListener(settings.getMainWindowVariable());
        }

        MenuItem action = new MenuItem();
        // Check to see if we should offer the enable or disable menu item
        if (state) {
            action.setLabel(RB("disable"));
            // action.setIcon(new ImageIcon(JMainMenu.class.getResource("icons/22/button_cancel.png")));
            action.setActionCommand("disable:" + name);
        } else {
            action.setLabel(RB("enable"));
            // action.setIcon(new ImageIcon(JMainMenu.class.getResource("icons/22/button_ok.png")));
            action.setActionCommand("enable:" + name);
        }

        action.addActionListener(settings.getMainWindowVariable());

        MenuItem desc = new MenuItem(RB("description"));
        desc.setActionCommand("description:" + name);
        desc.addActionListener(settings.getMainWindowVariable());

        tempName.add(prop);
        tempName.add(action);
        tempName.addSeparator();
        tempName.add(desc);

        tWPlugInMenu.add(tempName);

        /*
        MenuItem tempName = new MenuItem(name);
        tempName.setEnabled(state);
        tempName.setActionCommand("plugin:" + name);
        // tWPlugInMenu.add(name);
        tWPlugInMenu.add(tempName);
        */
    }

    public void updateConnectionMenu() {
        MuckMain mainVar = settings.getMainWindowVariable();

        // First, check to see if we have any connections.
        // If not, then we don't need to do any more here
        CHandler connHandler = settings.getConnectionHandler();
        if (connHandler.totalConnections() < 1) {
            return;
        }

        // Clear the current list of MU*'s
        tWMUListMenu.removeActionListener(mainVar);
        tWMUListMenu.removeAll();

        // Add the two generic entries... the next and previous choices
        StringBuffer tempName;
        MenuItem tempMenu;
        MenuShortcut shortCut;

        tempMenu = new MenuItem(RB("previousMU"));
        shortCut = new MenuShortcut(KeyEvent.VK_PAGE_DOWN, false);
        tempMenu.setShortcut(shortCut);
        tempMenu.setActionCommand(RB("previousMU"));
        if (connHandler.totalConnections() < 2) {
            tempMenu.setEnabled(false);
        }
        tWMUListMenu.add(tempMenu);

        tempMenu = new MenuItem(RB("nextMU"));
        shortCut = new MenuShortcut(KeyEvent.VK_PAGE_UP, false);
        tempMenu.setShortcut(shortCut);
        tempMenu.setActionCommand(RB("nextMU"));
        if (connHandler.totalConnections() < 2) {
            tempMenu.setEnabled(false);
        }
        tWMUListMenu.add(tempMenu);

        tWMUListMenu.addSeparator();

        int tc = connHandler.totalConnections() - 1;
        int activeConnection = connHandler.getActiveMUIndex();

        // Loop through our connections and build a new menu
        for (int i = 0; i <= tc; i++) {
            tempName = new StringBuffer(connHandler.getTitle(i));
            if (i == activeConnection) {
                // This is our active MU*
                // We'll just differentiate with a check beside it's name
                tWMUListMenu.add(new CheckboxMenuItem(tempName.toString(), true));
            } else {
                tempMenu = new MenuItem(tempName.toString());
                switch(i) {
                case 0:
                    shortCut = new MenuShortcut(KeyEvent.VK_1, false);
                    break;
                case 1:
                    shortCut = new MenuShortcut(KeyEvent.VK_2, false);
                    break;
                case 2:
                    shortCut = new MenuShortcut(KeyEvent.VK_3, false);
                    break;
                case 3:
                    shortCut = new MenuShortcut(KeyEvent.VK_4, false);
                    break;
                case 4:
                    shortCut = new MenuShortcut(KeyEvent.VK_5, false);
                    break;
                case 5:
                    shortCut = new MenuShortcut(KeyEvent.VK_6, false);
                    break;
                case 6:
                    shortCut = new MenuShortcut(KeyEvent.VK_7, false);
                    break;
                case 7:
                    shortCut = new MenuShortcut(KeyEvent.VK_8, false);
                    break;
                case 8:
                    shortCut = new MenuShortcut(KeyEvent.VK_9, false);
                    break;
                case 9:
                    shortCut = new MenuShortcut(KeyEvent.VK_0, false);
                    break;
                }

                tempMenu.setShortcut(shortCut);

                tempMenu.setActionCommand("ChangeMU:" + i);
                tempMenu.addActionListener(mainVar);
                tWMUListMenu.add(tempMenu);
            }
        }

        tWMUListMenu.addActionListener(mainVar);

    }

    public boolean isAutoFocus() {
        return tWAutoFocus.getState();
    }

    public boolean isReleasePause() {
        return tWReleasePauseItem.getState();
    }

    private String RB(String itemTarget) {
        return reader.LangString(anecho.JamochaMUD.JMConfig.BUNDLEBASE, itemTarget);
    }

    /** Set the menus to reflect if we're connected (true) or not (false) */
    public void setConnected(boolean state) {
        // This changes the flags on the MuckMain menu to 'connected'
        // Connect to MU*
        // Shouldn't the connect to MU* menu always be available?
        // cTM.setEnabled(!state);
        // Disconnect from MU*
        dFM.setEnabled(state);
        // Reconnect to MU*
        rTM.setEnabled(!state);
    }

}