/** JMappedComboBox.java is a GUI class that extends JComboBox
 * to allow a hidden "column" of information to go along with
 * the user-displayed information.  This is useful for keeping
 * "keys" to database entries or the like
 * Created on October 31, 2004, 12:51 PM
 */


package anecho.gui;

import java.util.Vector;
import javax.swing.ComboBoxModel;

/**
 *
 * @author  Jeff Robinson
 */
public class JMappedComboBox extends javax.swing.JComboBox {
    
        // private boolean useMap = false;     // Should our mapping feature be used?
    private Vector itemMap = new Vector(0);     // A list of our "keys" for our items

    private static final boolean DEBUG = false;
    // Constructors
    public JMappedComboBox() {
        super();
    }

    public JMappedComboBox(ComboBoxModel aModel) {
        super(aModel);
    }

    public JMappedComboBox(Object[] items) {
        this(items, new Object[items.length]);
    }

    public JMappedComboBox(Vector items) {
        this(items, new Vector(items.size()));
    }

    public JMappedComboBox(Object[] items, Object[] maps) {
        super(items);
        itemMap.removeAllElements();

        int mapSize = maps.length;

        for (int i = 0; i < mapSize; i++) {
            itemMap.addElement(maps[i]);
            if (DEBUG) {
                System.err.println("Adding: " + maps[i]);
            }
        }
    }

    public JMappedComboBox(Vector items, Vector maps) {
        super(items);

        itemMap.removeAllElements();
        itemMap = maps;
    }

    public void addItem(Object item) {
        this.addItem(item, null);
    }

    public void addItem(Object item, Object map) {
        super.addItem(item);
        itemMap.addElement(map);

        if (DEBUG) {
            System.err.println("Added item " + item + " with map: " + map);
        }
    }

    /** Return our map key based on the given index */
    public Object getMapAt(final int index) {
        Object retObj = new Object();

        if (index > 0 || index < itemMap.size()) {
            retObj = itemMap.elementAt(index);
        }

        return retObj;
    }

    public Object getSelectedMap() {
        Object retObj;
        int index = this.getSelectedIndex();
        
        if (index >= 0 && index < itemMap.size()) {
            retObj = itemMap.elementAt(index);
        } else {
            // retObj = new Object();
            retObj = null;
        }

        return retObj;
    }
    
    /*
    public int getSelectedIntMap() {
        int retInt;
        
        int index = this.getSelectedIndex();
        Object retObj = itemMap.elementAt(index);
        
        retInt = Integer.parseInt((String)itemMap.elementAt(index));
        
        return retInt;
    }
    */
    
    public void insertItemAt(Object anObject, int index) {
        this.insertItemAt(anObject, index, null);
    }

    public void insertItemAt(Object anObject, int index, Object map) {
        super.insertItemAt(anObject, index);
        itemMap.insertElementAt(map, index);
    }

    public void removeAllItems() {
        super.removeAllItems();
        itemMap.removeAllElements();
    }

    public void removeItem(Object anObject) {
        // int index = this.get
        // This may be ugly where we have to iterate through the whole list
        // of the JComboBox to find the index of our object...!

        // Fix Me XXX
    }

    public void removeItemAt(int index) {
        super.removeItemAt(index);
        itemMap.removeElementAt(index);
    }
    
    /** Creates new form BeanForm */
    /*
    public JMappedComboBox() {
        initComponents();
    }
    */

    public void setSelectedMap(Object map) {
        int index = 0;
        int mapItems = itemMap.size();
        String strMap = map.toString();
        
        for (int i = 0; i < mapItems; i++) {
            // if (itemMap.elementAt(i) == map) {
            if (itemMap.elementAt(i).toString().equals(strMap)) {
                if (DEBUG) {
                    System.err.println("JMappedComboBox has found a match to " + map);
                }
                index = i;
            }
        }
        
        this.setSelectedIndex(index);
       
    }
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        
    }//GEN-END:initComponents
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables
    
}
